@@ -11,7 +11,7 @@
 #' used to compute the IP weights. This defaults to \code{logit_integrand}, 
 #' which calculates the product of inverse logits for individuals in a group: 
 #' \eqn{\prod_{j = 1}^{n_i} \{r \times h_{ij}(b_i)^{A_{ij}}\}\{1 - r \times 
-#' h_{ij}(b_i)^{1 - A_{ij}} \} f_b(b_i; \theta_s)}{prod(r * plogis(X * fixef + b)^A * 
+#' h_{ij}(b_i)\}^{1 - A_{ij}} f_b(b_i; \theta_s)}{prod(r * plogis(X * fixef + b)^A * 
 #' (1 - r * plogis(X * fixef+ b))^(1 - A)) * 
 #' dnorm(sd = sqrt(ranef))} where \deqn{h_{ij}(b_i) = logit^{-1}
 #' (\mathbf{X}_{ij}\theta_a + b_i)} and \eqn{b_i} is a group-level random effect, 
@@ -25,7 +25,7 @@
 #' defines the log likelihood of the logit model used for \code{robust} variance
 #' estimation. Generally, this will be the same function as 
 #' \code{propensity_integrand}. Indeed, this is the default.
-#' @param allocations a vector of values in [0, 1]. Increasing the number of 
+#' @param allocations a vector of values in (0, 1). Increasing the number of 
 #' elements of the allocation vector greatly increases computation time; however, 
 #' a larger number of allocations will make plots look nicer. A minimum of two 
 #' allocations is required.
@@ -43,22 +43,20 @@
 #' details on how to pass the oracle parameters. 
 #' @param model_options a list of options passed to the function in 
 #' \code{model_method}. Defaults to \code{list(family = binomial(link = 'logit'))}. 
-#' When \code{model_method = 'oracle'}, the list must have two elements 
-#' \code{fixed.effects} and \code{random.effects}. If the model did not include 
+#' When \code{model_method = 'oracle'}, the list must have two elements (1)
+#' \code{fixed_effects} and (2) \code{random_effects}. If the model did not include 
 #' random effects, set \code{random.effects = NULL}.
 #' @param causal_estimation_method currently only supports \code{'ipw'}.
-#' @param causal_estimation_options A list with two slots. (1) \code{variance_estimation} is 
-#' either \code{'naive'} or \code{'robust'}. See details. Defaults to \code{'robust'}. (2) 
-#' is \code{set_NA_to_0}. Defaults to \code{TRUE}. When, for example, group sizes 
-#' reach over 1000, the product terms of the propensity diminish to zero. 
-#' This may result in \code{NaN} values for the weights or loglihood. This option
-#' sets such cases to zero.
+#' @param causal_estimation_options A list. Current options are: (1) \code{variance_estimation} is 
+#' either \code{'naive'} or \code{'robust'}. See details. Defaults to \code{'robust'}. 
 #' @param conf.level level for confidence intervals. Defaults to \code{0.95}.
 #' @param rescale.factor a scalar multiplication factor by which to rescale outcomes
 #' and effects. Defaults to \code{1}.
+#' @param runSilent if FALSE, status of computations are printed to console. Defaults to TRUE.
 #' @param ... Used to pass additional arguments to internal functions such as 
 #' \code{numDeriv::grad()} or \code{integrate()}. Additionally, arguments can be 
 #' passed to the \code{propensity_integrand} and \code{loglihood_integrand} functions.
+#' @inheritParams wght_calc
 #'
 #' @details The following formula includes a random effect for the group: \code{outcome | 
 #' exposure ~ propensity covariates + (1|group) | group}. In this instance, the 
@@ -83,8 +81,11 @@
 #' \code{variance_estimatation = 'robust'}. Refer to the web appendix of
 #' \href{http://dx.doi.org/10.1111/biom.12184}{Heydrich-Perez et al. (2014)} 
 #' for complete details.
+#' @references Perez-Heydrich, C., Hudgens, M. G., Halloran, M. E., Clemens, J. D., Ali, M., & Emch, M. E. (2014). 
+#' Assessing effects of cholera vaccination in the presence of interference. Biometrics, 70(3), 731-741.
 #' 
-#' 
+#' Tchetgen Tchetgen, E. J., & VanderWeele, T. J. (2012). On causal inference in the presence of interference. 
+#' Statistical Methods in Medical Research, 21(1), 55-75.
 #' @return Returns a list of overall and group-level IPW point estimates, overall 
 #' and group-level IPW point estimates (using the weight derivatives), derivatives
 #' of the loglihood, the computed weight matrix, the computed 
@@ -100,12 +101,13 @@ interference <- function(formula,
                          allocations,
                          data,
                          model_method = "glmer",
-                         model_options = list(family = binomial(link = 'logit')),
+                         model_options = list(family = stats::binomial(link = 'logit')),
                          causal_estimation_method = 'ipw',
-                         causal_estimation_options = list(set_NA_to_0 = TRUE, 
-                                                          variance_estimation = 'robust'),
-                         conf.level = 0.95,
-                         rescale.factor = 1,   
+                         causal_estimation_options = list(variance_estimation = 'robust'),
+                         conf.level     = 0.95,
+                         rescale.factor = 1,
+                         integrate_allocation = TRUE,
+                         runSilent      = TRUE, 
                          ...)
 {
   ## Necessary bits ##
@@ -116,9 +118,10 @@ interference <- function(formula,
   cformula        <- Formula::Formula(formula)
   len_lhs         <- length(cformula)[1]
   len_rhs         <- length(cformula)[2]
+  data            <- as.data.frame(data) # make sure data is data.frame not data.table, etc.
 
   ## For the sake of consistency downstream, reorder data frame by groups ##
-  group_var <- attr(terms(cformula, lhs = 0, rhs = len_rhs), 'term.labels')
+  group_var <- attr(stats::terms(cformula, lhs = 0, rhs = len_rhs), 'term.labels')
   data <- data[order(data[ , group_var]), ]
   
   ## Parse out the formula into necessary pieces ##
@@ -133,7 +136,7 @@ interference <- function(formula,
     B <- A
   }
   
-  propensity_formula <- formula(terms(cformula, lhs = len_lhs, rhs = -2))
+  propensity_formula <- formula(stats::terms(cformula, lhs = len_lhs, rhs = -2))
   random.count <- length(lme4::findbars(propensity_formula))
 
   trt_lvls     <- sort(unique(A))
@@ -152,7 +155,7 @@ interference <- function(formula,
   }
   
   if(min(allocations) < 0 | max(allocations) > 1){
-    stop('Allocations must be between 0 and 1 (inclusive)')
+    stop('Allocations must be between 0 and 1')
   }
   
   if(length(allocations) < 2){
@@ -163,27 +166,38 @@ interference <- function(formula,
     stop('The group variable must have at least 2 groups (more is better).')
   }
   
+  if(!(causal_estimation_options$variance_estimation %in% c('naive', 'robust'))){
+    stop("The variance estimation method should be 'naive' or 'robust'")
+  }
+  
   #### Compute Parameter Estimates ####
 
   estimation_args <- append(list(formula = propensity_formula, data = data), 
                             model_options)
   
+  parameters <- list()
+  
   if(model_method == "glmer"){
     propensity_model <- do.call(lme4::glmer, args = estimation_args)
-    fixed.effects  <- lme4::getME(propensity_model, 'fixef')
-    random.effects <- lme4::getME(propensity_model, 'theta')
+    parameters$fixed_effects  <- lme4::getME(propensity_model, 'fixef')
+    parameters$random_effects <- lme4::getME(propensity_model, 'theta')
     X <- lme4::getME(propensity_model, "X")
+    
+    if(sum(parameters$random_effects == 0) > 0){
+      stop('At least one random effect was estimated as 0. This will lead to a
+           non-invertible matrix if using robust variance estimation.')
+    }
   } else if(model_method == "glm"){
     propensity_model <- do.call("glm", args = estimation_args)
-    fixed.effects  <- coef(propensity_model)
-    random.effects <- NULL
-    X <- model.matrix(propensity_model)
+    parameters$fixed_effects  <- stats::coef(propensity_model)
+    parameters$random_effects <- NULL
+    X <- stats::model.matrix(propensity_model)
   } else if(model_method == "oracle"){
-    fixed.effects  <- model_options[[1]]
-    random.effects <- model_options[[2]]
-    X <- model.matrix(propensity_formula, data)
+    parameters$fixed_effects  <- model_options[[1]]
+    parameters$random_effects <- model_options[[2]]
+    X <- stats::model.matrix(propensity_formula, data)
     
-    if(length(fixed.effects) != ncol(X)){
+    if(length(parameters$fixed_effects) != ncol(X)){
       stop('oracle fixed effects vector must have length of # of columns of X')
     }
   }
@@ -199,14 +213,15 @@ interference <- function(formula,
                        list(propensity_integrand = integrandFUN, 
                             loglihood_integrand  = likelihoodFUN,
                             allocations          = allocations,
-                            fixed.effects        = fixed.effects, 
-                            random.effects       = random.effects,
+                            parameters           = unlist(parameters),
+                            runSilent            = runSilent, 
+                            integrate_allocation = integrate_allocation,
                             Y = Y, X = X, A = A, B = B, G = G))
   
     ipw <- do.call(ipw_interference, args = ipw_args)
     out <- append(out, ipw)
     
-    print('Computing effect estimates...')
+    if(runSilent != TRUE){print('Computing effect estimates...')} #BB 2015-06-23
     
     estimate_args <- list(obj = ipw,
                           variance_estimation = causal_estimation_options$variance_estimation,
@@ -245,7 +260,7 @@ interference <- function(formula,
                       conf.level   = conf.level,
                       ngroups      = N, 
                       nallocations = k,
-                      npredictors  = length(fixed.effects),
+                      npredictors  = length(parameters$fixed_effects),
                       ntreatments  = l,
                       allocations  = allocations,
                       treatments   = trt_lvls,
@@ -254,6 +269,6 @@ interference <- function(formula,
   
   class(out) <- "interference"
   
-  print('Interference complete')
+  if(runSilent != TRUE){print('Interference complete')} #BB 2015-06-23
   return(out)
 }
\ No newline at end of file
